@@ -105,6 +105,7 @@ void gaa_help()
 	printf("Mcrypt help\nUsage : mcrypt [options] file1 file2 ...""\n");
 	__gaa_helpsingle('V', "verbose", "", "some more information is displayed.");
 	__gaa_helpsingle('q', "quiet", "", "suppress some non critical warnings.");
+	__gaa_helpsingle('t', "time", "", "Print timing information.");
 	__gaa_helpsingle('F', "force", "", "forces output to stdout.");
 	__gaa_helpsingle('r', "random", "", "Use real random data (if your system supports it).");
 	__gaa_helpsingle('b', "bare", "", "Do not keep algorithm information in the encrypted""file.");
@@ -148,64 +149,66 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 93 "mcrypt.gaa"
+#line 96 "mcrypt.gaa"
 	char **input;
-#line 92 "mcrypt.gaa"
+#line 95 "mcrypt.gaa"
 	int size;
-#line 83 "mcrypt.gaa"
+#line 86 "mcrypt.gaa"
 	char *hash;
-#line 79 "mcrypt.gaa"
+#line 82 "mcrypt.gaa"
 	char *mode;
-#line 74 "mcrypt.gaa"
+#line 77 "mcrypt.gaa"
 	char **keys;
-#line 73 "mcrypt.gaa"
+#line 76 "mcrypt.gaa"
 	int keylen;
-#line 70 "mcrypt.gaa"
+#line 73 "mcrypt.gaa"
 	char *algorithm;
-#line 67 "mcrypt.gaa"
+#line 70 "mcrypt.gaa"
 	char *config_file;
-#line 66 "mcrypt.gaa"
+#line 69 "mcrypt.gaa"
 	int config;
-#line 63 "mcrypt.gaa"
+#line 66 "mcrypt.gaa"
 	char *keyfile;
-#line 59 "mcrypt.gaa"
+#line 62 "mcrypt.gaa"
 	int unlink_flag;
-#line 56 "mcrypt.gaa"
+#line 59 "mcrypt.gaa"
 	char *kmode;
-#line 52 "mcrypt.gaa"
+#line 55 "mcrypt.gaa"
 	char *modes_directory;
-#line 49 "mcrypt.gaa"
+#line 52 "mcrypt.gaa"
 	char *algorithms_directory;
-#line 46 "mcrypt.gaa"
+#line 49 "mcrypt.gaa"
 	int keysize;
-#line 43 "mcrypt.gaa"
+#line 46 "mcrypt.gaa"
 	int ed_specified;
-#line 42 "mcrypt.gaa"
+#line 45 "mcrypt.gaa"
 	int ein;
-#line 41 "mcrypt.gaa"
+#line 44 "mcrypt.gaa"
 	int din;
-#line 38 "mcrypt.gaa"
+#line 41 "mcrypt.gaa"
 	int double_check;
-#line 35 "mcrypt.gaa"
+#line 38 "mcrypt.gaa"
 	int flush;
-#line 31 "mcrypt.gaa"
+#line 34 "mcrypt.gaa"
 	int nodelete;
-#line 28 "mcrypt.gaa"
+#line 31 "mcrypt.gaa"
 	int noiv;
-#line 25 "mcrypt.gaa"
+#line 28 "mcrypt.gaa"
 	int noecho;
-#line 22 "mcrypt.gaa"
+#line 25 "mcrypt.gaa"
 	int nolock;
-#line 19 "mcrypt.gaa"
+#line 22 "mcrypt.gaa"
 	int bzipflag;
-#line 16 "mcrypt.gaa"
+#line 19 "mcrypt.gaa"
 	int gzipflag;
-#line 13 "mcrypt.gaa"
+#line 16 "mcrypt.gaa"
 	int bare_flag;
-#line 10 "mcrypt.gaa"
+#line 13 "mcrypt.gaa"
 	int real_random_flag;
-#line 7 "mcrypt.gaa"
+#line 10 "mcrypt.gaa"
 	int force;
+#line 7 "mcrypt.gaa"
+	int timer;
 #line 3 "mcrypt.gaa"
 	int quiet;
 
@@ -262,7 +265,7 @@ int gaa_error = 0;
 #define GAA_MULTIPLE_OPTION     3
 
 #define GAA_REST                0
-#define GAA_NB_OPTION           31
+#define GAA_NB_OPTION           32
 #define GAAOPTID_license	1
 #define GAAOPTID_version	2
 #define GAAOPTID_help	3
@@ -292,8 +295,9 @@ int gaa_error = 0;
 #define GAAOPTID_bare	27
 #define GAAOPTID_random	28
 #define GAAOPTID_force	29
-#define GAAOPTID_quiet	30
-#define GAAOPTID_verbose	31
+#define GAAOPTID_time	30
+#define GAAOPTID_quiet	31
+#define GAAOPTID_verbose	32
 
 #line 168 "gaa.skel"
 
@@ -361,7 +365,7 @@ num = 0; \
 if(!OK) \
 { \
 num = 0;                                                            \
-target = _mcrypt_malloc((gaa_last_non_option - gaa_index) * sizeof(type));  \
+target = malloc((gaa_last_non_option - gaa_index) * sizeof(type));  \
 for(; gaa_index < gaa_last_non_option; gaa_index++)                 \
 {                                                                   \
     if(gaa_arg_used[gaa_index] == 0)                                \
@@ -378,7 +382,7 @@ if(num == 0)                                                        \
 if(!OK) \
 { \
 num = 0;                                                            \
-target = _mcrypt_malloc((gaa_last_non_option - gaa_index) * sizeof(type));  \
+target = malloc((gaa_last_non_option - gaa_index) * sizeof(type));  \
 for(; gaa_index < gaa_last_non_option; gaa_index++)                 \
 {                                                                   \
     if(gaa_arg_used[gaa_index] == 0)                                \
@@ -603,6 +607,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECK1STR("b", GAAOPTID_bare);
 			GAA_CHECK1STR("r", GAAOPTID_random);
 			GAA_CHECK1STR("F", GAAOPTID_force);
+			GAA_CHECK1STR("t", GAAOPTID_time);
 			GAA_CHECK1STR("q", GAAOPTID_quiet);
 			GAA_CHECK1STR("V", GAAOPTID_verbose);
 
@@ -638,6 +643,7 @@ int gaa_get_option_num(char *str, int status)
 			GAA_CHECKSTR("bare", GAAOPTID_bare);
 			GAA_CHECKSTR("random", GAAOPTID_random);
 			GAA_CHECKSTR("force", GAAOPTID_force);
+			GAA_CHECKSTR("time", GAAOPTID_time);
 			GAA_CHECKSTR("quiet", GAAOPTID_quiet);
 			GAA_CHECKSTR("verbose", GAAOPTID_verbose);
 
@@ -684,42 +690,42 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
     {
 	case GAAOPTID_license:
 	OK = 0;
-#line 90 "mcrypt.gaa"
+#line 93 "mcrypt.gaa"
 { mcrypt_license(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_version:
 	OK = 0;
-#line 89 "mcrypt.gaa"
+#line 92 "mcrypt.gaa"
 { mcrypt_version(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_help:
 	OK = 0;
-#line 88 "mcrypt.gaa"
+#line 91 "mcrypt.gaa"
 { usage(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_hashlist:
 	OK = 0;
-#line 87 "mcrypt.gaa"
+#line 90 "mcrypt.gaa"
 { print_hashlist(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_keymodeslist:
 	OK = 0;
-#line 86 "mcrypt.gaa"
+#line 89 "mcrypt.gaa"
 { print_keylist(); exit(0); ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_list:
 	OK = 0;
-#line 85 "mcrypt.gaa"
+#line 88 "mcrypt.gaa"
 { print_list(); exit(0); ;};
 
 		return GAA_OK;
@@ -729,7 +735,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_hash.arg1, gaa_getstr, GAATMP_hash.size1);
 		gaa_index++;
-#line 84 "mcrypt.gaa"
+#line 87 "mcrypt.gaa"
 { gaaval->hash = GAATMP_hash.arg1 ;};
 
 		return GAA_OK;
@@ -739,7 +745,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_mode.arg1, gaa_getstr, GAATMP_mode.size1);
 		gaa_index++;
-#line 80 "mcrypt.gaa"
+#line 83 "mcrypt.gaa"
 { gaaval->mode = GAATMP_mode.arg1 ;};
 
 		return GAA_OK;
@@ -747,7 +753,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 	case GAAOPTID_key:
 	OK = 0;
 		GAA_LIST_FILL(GAATMP_key.arg1, gaa_getstr, char*, GAATMP_key.size1);
-#line 75 "mcrypt.gaa"
+#line 78 "mcrypt.gaa"
 { gaaval->keys = GAATMP_key.arg1; gaaval->keylen = GAATMP_key.size1 ;};
 
 		return GAA_OK;
@@ -757,7 +763,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithm.arg1, gaa_getstr, GAATMP_algorithm.size1);
 		gaa_index++;
-#line 71 "mcrypt.gaa"
+#line 74 "mcrypt.gaa"
 { gaaval->algorithm = GAATMP_algorithm.arg1 ;};
 
 		return GAA_OK;
@@ -767,7 +773,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_config.arg1, gaa_getstr, GAATMP_config.size1);
 		gaa_index++;
-#line 68 "mcrypt.gaa"
+#line 71 "mcrypt.gaa"
 { gaaval->config=1; if (GAATMP_config.size1==1) gaaval->config_file = GAATMP_config.arg1 ;};
 
 		return GAA_OK;
@@ -777,14 +783,14 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keyfile.arg1, gaa_getstr, GAATMP_keyfile.size1);
 		gaa_index++;
-#line 64 "mcrypt.gaa"
+#line 67 "mcrypt.gaa"
 { gaaval->keyfile = GAATMP_keyfile.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_unlink:
 	OK = 0;
-#line 60 "mcrypt.gaa"
+#line 63 "mcrypt.gaa"
 { gaaval->unlink_flag = 1 ;};
 
 		return GAA_OK;
@@ -794,7 +800,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keymode.arg1, gaa_getstr, GAATMP_keymode.size1);
 		gaa_index++;
-#line 57 "mcrypt.gaa"
+#line 60 "mcrypt.gaa"
 { gaaval->kmode = GAATMP_keymode.arg1 ;};
 
 		return GAA_OK;
@@ -804,7 +810,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_modes_directory.arg1, gaa_getstr, GAATMP_modes_directory.size1);
 		gaa_index++;
-#line 53 "mcrypt.gaa"
+#line 56 "mcrypt.gaa"
 { gaaval->modes_directory = GAATMP_modes_directory.arg1 ;};
 
 		return GAA_OK;
@@ -814,7 +820,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_algorithms_directory.arg1, gaa_getstr, GAATMP_algorithms_directory.size1);
 		gaa_index++;
-#line 50 "mcrypt.gaa"
+#line 53 "mcrypt.gaa"
 { gaaval->algorithms_directory = GAATMP_algorithms_directory.arg1 ;};
 
 		return GAA_OK;
@@ -824,93 +830,100 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		GAA_TESTMOREARGS;
 		GAA_FILL(GAATMP_keysize.arg1, gaa_getint, GAATMP_keysize.size1);
 		gaa_index++;
-#line 47 "mcrypt.gaa"
+#line 50 "mcrypt.gaa"
 { gaaval->keysize = GAATMP_keysize.arg1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_decrypt:
 	OK = 0;
-#line 44 "mcrypt.gaa"
+#line 47 "mcrypt.gaa"
 { gaaval->ed_specified=1; gaaval->din = 1; gaaval->ein=0; ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_doublecheck:
 	OK = 0;
-#line 39 "mcrypt.gaa"
+#line 42 "mcrypt.gaa"
 { gaaval->double_check = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_flush:
 	OK = 0;
-#line 36 "mcrypt.gaa"
+#line 39 "mcrypt.gaa"
 { gaaval->flush = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_nodelete:
 	OK = 0;
-#line 32 "mcrypt.gaa"
+#line 35 "mcrypt.gaa"
 { gaaval->nodelete = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_noiv:
 	OK = 0;
-#line 29 "mcrypt.gaa"
+#line 32 "mcrypt.gaa"
 { gaaval->noiv = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_echo:
 	OK = 0;
-#line 26 "mcrypt.gaa"
+#line 29 "mcrypt.gaa"
 { gaaval->noecho = 0 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_nolock:
 	OK = 0;
-#line 23 "mcrypt.gaa"
+#line 26 "mcrypt.gaa"
 { gaaval->nolock = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_bzip2:
 	OK = 0;
-#line 20 "mcrypt.gaa"
+#line 23 "mcrypt.gaa"
 { gaaval->bzipflag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_gzip:
 	OK = 0;
-#line 17 "mcrypt.gaa"
+#line 20 "mcrypt.gaa"
 { gaaval->gzipflag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_bare:
 	OK = 0;
-#line 14 "mcrypt.gaa"
+#line 17 "mcrypt.gaa"
 { gaaval->bare_flag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_random:
 	OK = 0;
-#line 11 "mcrypt.gaa"
+#line 14 "mcrypt.gaa"
 { gaaval->real_random_flag = 1 ;};
 
 		return GAA_OK;
 		break;
 	case GAAOPTID_force:
 	OK = 0;
-#line 8 "mcrypt.gaa"
+#line 11 "mcrypt.gaa"
 { gaaval->force = 1 ;};
 
+		return GAA_OK;
+		break;
+	case GAAOPTID_time:
+	OK = 0;
+#line 8 "mcrypt.gaa"
+{ gaaval->timer = 1 ;};
+
 		return GAA_OK;
 		break;
 	case GAAOPTID_quiet:
@@ -929,7 +942,7 @@ int gaa_try(int gaa_num, int gaa_index, gaainfo *gaaval, char *opt_list)
 		break;
 	case GAA_REST:
 		GAA_OPTIONALLIST_FILL(GAAREST_tmp.arg1, gaa_getstr, char*, GAAREST_tmp.size1);
-#line 94 "mcrypt.gaa"
+#line 97 "mcrypt.gaa"
 { gaaval->input = GAAREST_tmp.arg1; gaaval->size = GAAREST_tmp.size1 ;};
 
 		return GAA_OK;
@@ -950,14 +963,14 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
     GAAargv = argv;
     GAAargc = argc;
 
-    opt_list = (char*) _mcrypt_malloc(GAA_NB_OPTION + 1);
+    opt_list = (char*) malloc(GAA_NB_OPTION + 1);
     for(i = 0; i < GAA_NB_OPTION + 1; i++)
         opt_list[i] = 0;
     /* initialization */
     if(inited == 0)
     {
 
-#line 96 "mcrypt.gaa"
+#line 99 "mcrypt.gaa"
 { gaaval->force=0; gaaval->quiet=1; gaaval->real_random_flag=0; gaaval->config=0;
        gaaval->bare_flag = 0; gaaval->din = 0; gaaval->ein=1; gaaval->unlink_flag = 0;
        gaaval->kmode = NULL; gaaval->keylen=0; gaaval->gzipflag=0; gaaval->bzipflag=0;
@@ -965,12 +978,12 @@ int gaa(int argc, char **argv, gaainfo *gaaval)
        gaaval->config_file=NULL; gaaval->mode=NULL; gaaval->input=NULL; gaaval->ed_specified=0;
        gaaval->double_check=0; gaaval->nolock=0; gaaval->noecho=1; gaaval->flush=0; gaaval->keysize=0;
        gaaval->algorithms_directory=NULL; gaaval->modes_directory=NULL; gaaval->nodelete=0;
-       gaaval->hash=NULL ;};
+       gaaval->hash=NULL; gaaval->timer=0 ;};
 
     }
     inited = 1;
 #line 438 "gaa.skel"
-    gaa_arg_used = _mcrypt_malloc(argc * sizeof(char));
+    gaa_arg_used = malloc(argc * sizeof(char));
     for(i = 1; i < argc; i++)
         gaa_arg_used[i] = 0;
     for(i = 1; i < argc; i++)
@@ -1066,8 +1079,8 @@ if(gaa_processing_file == 0)
             return 0;
         }
     }
-    _mcrypt_free(gaa_arg_used);
-    _mcrypt_free(opt_list);
+    free(gaa_arg_used);
+    free(opt_list);
     return -1;
 }
 
@@ -1109,7 +1122,7 @@ int gaa_internal_get_next_str(FILE *file, gaa_str_node *tmp_str, int argc)
         if(fscanf(file,"%c", &a) != 1) a = ' ';
     }
     
-    tmp_str->str = _mcrypt_malloc((len + 1) * sizeof(char));
+    tmp_str->str = malloc((len + 1) * sizeof(char));
     if(newline == 1)
     {
         tmp_str->str[0] = '-';
@@ -1161,7 +1174,7 @@ int gaa_file(char *name, gaainfo *gaaval)
     do
     {
         argc++;
-        *tmp_str = _mcrypt_malloc(sizeof(gaa_str_node));
+        *tmp_str = malloc(sizeof(gaa_str_node));
         (*tmp_str)->str = NULL;
         (*tmp_str)->next = NULL;
 
@@ -1173,7 +1186,7 @@ int gaa_file(char *name, gaainfo *gaaval)
     if(rval == 1)
         return 0;
     
-    argv = _mcrypt_malloc((1 + argc) * sizeof(char*));
+    argv = malloc((1 + argc) * sizeof(char*));
     tmp_str2 = first_str;
     argv[0] = "cfg";
     for(i = 1; i < argc; i++)
