@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mcrypt.c,v 1.14 2001/09/24 14:11:10 nmav Exp $ */
+/* $Id: mcrypt.c,v 1.18 2002/01/26 09:09:54 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -36,8 +36,14 @@
 #include <environ.h>
 #include <popen.h>
 
+#ifdef HAVE_FTIME
+# include <sys/timeb.h>
+#else
+# include <time.h>
+#endif
+
 static char rcsid[] =
-    "$Id: mcrypt.c,v 1.14 2001/09/24 14:11:10 nmav Exp $";
+    "$Id: mcrypt.c,v 1.18 2002/01/26 09:09:54 nmav Exp $";
 
 char tmperr[128];
 unsigned int stream_flag = FALSE;
@@ -62,6 +68,7 @@ int gzipflag = FALSE;
 int bzipflag = FALSE;
 #endif
 int flush = FALSE;
+int timer = FALSE;
 char *outfile = 0, *tmpc;
 char *algorithms_directory = NULL;
 char *modes_directory = NULL;
@@ -114,7 +121,8 @@ void usage(void)
 		 " -V --verbose                 more information is displayed.\n"
 		 " -b --bare                    Do not keep algorithm information in the encrypted\n"
 		 "                              file.\n"
-		 " -F --force                   forces output to stdout.\n"));
+		 " -F --force                   forces output to stdout.\n"
+		 " -t --time                    Prints timing information.\n"));
 
 #ifdef HAVE_DEV_RANDOM
 	fprintf(stdout,
@@ -163,6 +171,56 @@ void mcrypt_license()
 		  "Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 }
 
+/* Accepts time elapsed in milliseconds and file size in bytes.
+ */
+static void show_stats( time_t ms_time, size_t file_size) {
+time_t time = ms_time/1000;
+time_t millitm = ms_time % 1000;
+int show = 0;
+
+	fprintf(stderr, "Time needed: ");
+	
+	if (time/3600 > 0) {
+		fprintf(stderr, "%lu hours", time/3600);
+		show = 1;
+	}
+	
+	if ((time/60)-( (time/3600)*60 ) > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu minutes", (time/60)-(time/3600) );
+		show = 1;
+	}
+
+	if (time - ((time/60)+((time/3600)*60)*60 ) > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu seconds", time-((time/60)+(time/3600)) );
+		show = 1;
+	}
+	
+	if (millitm > 0) {
+		if (show>0) fprintf(stderr, ", ");
+		fprintf( stderr, "%lu milliseconds", millitm);
+		show = 1;
+	}
+	
+	if (show==0)
+		fprintf(stderr, "Wow, no time counted!\n");
+	else {
+		fprintf(stderr, ".\n");
+	
+		if (file_size > 1000000) {
+			fprintf(stderr, _("Encrypted %.3f MB, at a %.3f MB/sec rate (1 MB == 1000000 Bytes).\n"), 
+				(float)( (double)file_size/(double)1000000),
+				(float)( (( (double) file_size/ (double)1000000)/ (double)ms_time)*(double)1000));
+		} else { /* KB */
+			fprintf(stderr, _("Encrypted %.3f KB, at a %.3f KB/sec rate (1 KB == 1000 Bytes).\n"), 
+				(float)( (double)file_size/(double)1000),
+				(float)( (( (double) file_size/ (double)1000)/ (double)ms_time)*(double)1000));
+		}
+	}
+}
+
+
 int check_hash_algo(char *chain)
 {
 	int i;
@@ -333,7 +391,8 @@ int main(int argc, char **argv)
 	flush = info.flush;
 	nodelete = info.nodelete;
 	keysize = info.keysize;
-
+	timer = info.timer;
+	
 	algorithms_directory = info.algorithms_directory;
 	modes_directory = info.modes_directory;
 #ifdef ZIP
@@ -352,21 +411,17 @@ int main(int argc, char **argv)
 		strcpy(hash, info.hash);
 		hash_algorithm = check_hash_algo(hash);
 
-		if (hash_algorithm < 0) {
-			hash_algorithm = MHASH_CRC32;
-			fprintf(stderr,
-				_
-				("The '%s' hash algorithm was not found.\n"),
-				hash);
-			fprintf(stderr,
-				_
-				("Use the --hashlist parameter to view all supported hashes.\n"));
-			exit(1);
-		}
 	} else {
-		hash_algorithm = MHASH_CRC32;
-		hash = _mcrypt_malloc(strlen("crc32") + 1);
-		strcpy(hash, "crc32");
+		hash = _mcrypt_malloc(strlen(HASH_ALGORITHM)+1);
+		strcpy(hash, HASH_ALGORITHM);
+		hash_algorithm = check_hash_algo(hash);
+	}
+
+	if (hash_algorithm < 0) {
+		fprintf(stderr,
+			_("The '%s' hash algorithm was not found.\n"), hash);
+		fprintf(stderr, _("Use the --hashlist parameter to view all supported hashes.\n"));
+		exit(1);
 	}
 
 	if (info.keyfile != NULL) {
@@ -814,6 +869,14 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	word32 *IV = NULL;
 	word32 *salt = NULL;
 	int hash_size = mhash_get_block_size(hash_algorithm);
+#ifdef HAVE_FTIME
+	struct timeb start_time;
+	struct timeb end_time;
+#else
+	time_t start_time;
+	time_t end_time;
+#endif
+	size_t file_sum = 0;
 
 	word8 *ciphertext, *ciphertext_pad = NULL;
 	FILE *TOF = NULL;
@@ -834,7 +897,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	blocksize = mcrypt_enc_get_block_size(td);
 
 	ciphertext = _mcrypt_malloc(blocksize * BUFFER_SIZE);
-
+	
 	if (bare_flag == FALSE) {
 		salt = _mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
 
@@ -1004,6 +1067,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 
 
+#ifdef HAVE_FTIME
+	ftime(&start_time);
+#else
+	start_time = time(0);
+#endif
 
 	tc = mcrypt_generic_init(td, keyword, len, IV);
 	if (tc < 0) {
@@ -1019,7 +1087,8 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	/* read the first n bytes of the file and store to ciphertext */
 	for (;;) {
 		how = fread(ciphertext, 1, blocksize * BUFFER_SIZE, FROMF);
-
+		file_sum += how;
+		
 		if (how < BUFFER_SIZE * blocksize) {
 			if (ferror(FROMF) != 0) {
 				perror("fread");
@@ -1085,9 +1154,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		}
 	}
 
-	if (pad_size > 0)
+	if (pad_size > 0) {
 		ciphertext_pad = _mcrypt_malloc(pad_size);
-
+		if (ciphertext_pad==NULL)
+			err_quit(_("Memory error\n"));
+	}
 	if (mcrypt_enc_is_block_mode(td) == 1) {
 		memmove(ciphertext_pad,
 			&ciphertext[full_blocks * blocksize], rest_blocks);
@@ -1115,9 +1186,11 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 		}
 	}
 
-
-	if (flush == TRUE)
-		fflush(TOF);
+#ifdef HAVE_FTIME
+	ftime(&end_time);
+#else	
+	end_time = time(0);
+#endif
 
 	/* if bare_flag==FALSE */
 	/* End of copy */
@@ -1157,6 +1230,31 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	_mcrypt_free(ciphertext);
 
 
+	if (timer!=FALSE) {
+#ifdef HAVE_FTIME
+		end_time.time*=1000;
+		end_time.time+=end_time.millitm;
+
+		start_time.time*=1000;
+		start_time.time+=start_time.millitm;
+		
+		end_time.time -= start_time.time;
+		
+		start_time.time = end_time.time;
+		
+		end_time.millitm = end_time.time % 1000;
+
+		show_stats( end_time.time, file_sum);
+
+#else
+		end_time -= start_time;
+
+		show_stats( end_time, file_sum);
+
+#endif
+	}
+
+
 	return 0;
 }
 
@@ -1189,6 +1287,14 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	int j;
 	FILE *RTOF;
 	FILE *FROMF;
+#ifdef HAVE_FTIME
+	struct timeb start_time;
+	struct timeb end_time;
+#else
+	time_t start_time;
+	time_t end_time;
+#endif
+	size_t file_sum = 0;
 
 
 /* open files */
@@ -1249,7 +1355,6 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 
 	if (bare_flag == FALSE) {
-		salt = _mcrypt_malloc(SALT_SIZE);
 		if (check_file_head
 		    (FROMF, local_algorithm, local_mode, local_keymode,
 		     &keysize, local_salt, &salt_size) != 0) {
@@ -1334,6 +1439,11 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	fprintf(stderr, "\n");
 #endif
 
+#ifdef HAVE_FTIME
+	ftime(&start_time);
+#else
+	start_time = time(0);
+#endif
 
 	j = mcrypt_generic_init(td, keyword, len, IV);
 	if (j < 0) {
@@ -1369,6 +1479,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 			how =
 			    fread(ciphertext, 1, blocksize * BUFFER_SIZE,
 				  FROMF);
+			file_sum += how;
+
 			if (pad_size > how)
 				pad_size -= blocksize;
 			how -= pad_size;
@@ -1448,6 +1560,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 				    fread(&ciphertext[pad_size], 1,
 					  blocksize * BUFFER_SIZE -
 					  pad_size, FROMF);
+				file_sum += how;
+				
 				mdecrypt_generic(td, ciphertext, how);
 				memmove(ciphertext_p_old, &ciphertext[how],
 					pad_size);
@@ -1461,6 +1575,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 				how =
 				    fread(ciphertext, 1, buf_block, FROMF);
+				file_sum+=how;
 
 				if (how == buf_block && start == 1) {
 					if (fwrite
@@ -1570,6 +1685,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 				how =
 				    fread(ciphertext, 1, BUFFER_SIZE,
 					  FROMF);
+				file_sum += how;
 				mdecrypt_generic(td, ciphertext, how);
 
 				if (how != BUFFER_SIZE) {
@@ -1632,6 +1748,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 				how =
 				    fread(ciphertext, 1, BUFFER_SIZE,
 					  FROMF);
+				file_sum += how;
 				mdecrypt_generic(td, ciphertext, how);
 
 				if (how != BUFFER_SIZE) {
@@ -1665,6 +1782,13 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	}
 
 
+#ifdef HAVE_FTIME
+	ftime(&end_time);
+#else	
+	end_time = time(0);
+#endif
+
+
 /* close and unlock files */
 
 	if (stream_flag == FALSE) {
@@ -1757,6 +1881,30 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
 	mcrypt_generic_end(td);
 
+	if (timer!=FALSE) {
+#ifdef HAVE_FTIME
+		end_time.time*=1000;
+		end_time.time+=end_time.millitm;
+
+		start_time.time*=1000;
+		start_time.time+=start_time.millitm;
+		
+		end_time.time -= start_time.time;
+		
+		start_time.time = end_time.time;
+		
+		end_time.millitm = end_time.time % 1000;
+
+		show_stats( end_time.time, file_sum);
+
+#else
+		end_time -= start_time;
+
+		show_stats( end_time, file_sum);
+
+#endif
+	}
+
 	return 0;
 
 
